<?php

use Icinga\Web\Wizard;

?>
<form
 id="<?= $this->escape($form->getName()); ?>"
 name="<?= $this->escape($form->getName()); ?>"
 enctype="<?= $this->escape($form->getEncType()); ?>"
 method="<?= $this->escape($form->getMethod()); ?>"
 action="<?= $this->escape($form->getAction()); ?>"
 class="icinga-controls"
 data-progress-element="<?= Wizard::PROGRESS_ELEMENT; ?>"
>
<h2><?= $this->translate('Modules', 'setup.page.title'); ?></h2>
<p><?= $this->translate('The following modules were found in your Icinga Web 2 installation. To enable and configure a module, just tick it and click "Next".'); ?></p>
<?php foreach ($form->getElements() as $element): ?>
 <?php if (! in_array($element->getName(), array(Wizard::BTN_PREV, Wizard::BTN_NEXT, Wizard::PROGRESS_ELEMENT, $form->getTokenElementName(), $form->getUidElementName()))): ?>
  <div class="module">
    <div class="header">
      <h3><label for="<?= $element->getId(); ?>"><strong><?= $element->getLabel(); ?></strong></label></h3>
      <div class="element">
        <?= $element; ?>
      </div>
    </div>
    <label class="description" for="<?= $element->getId(); ?>"><?= $element->getDescription(); ?></label>
  </div>
 <?php endif ?>
<?php endforeach ?>
  <?= $form->getElement($form->getTokenElementName()); ?>
  <?= $form->getElement($form->getUidElementName()); ?>
  <?= $form->getDisplayGroup('buttons'); ?>
</form>
