<div class="content">
<h1><?= $this->translate("I'm ready to search, waiting for your input") ?></h1>
<p><strong><?= $this->translate('Hint') ?>: </strong><?= $this->translate(
    'Please use the asterisk (*) as a placeholder for wildcard searches.'
  . " For convenience I'll always add a wildcard in front and after your"
  . ' search string.'
) ?></p>
</div>
