/*! Icinga Web 2 | (c) 2014 Icinga GmbH | GPLv2+ */

// Ensure styling is light, exports use a white background

@gray: #7F7F7F;
@gray-semilight: #A9A9A9;
@gray-light: #C9C9C9;
@gray-lighter: #EEEEEE;
@gray-lightest: #F7F7F7;
@icinga-blue: #0095BF;
@low-sat-blue: #dae3e6;
@low-sat-blue-dark: #becbcf;
@body-bg-color: #fff;
@text-color: @black;
@text-color-light: @gray;
@tr-active-color: @body-bg-color;
@tr-hover-color: @body-bg-color;

// Page layout

@page {
  margin: 1cm;
}

body {
  font-family: sans-serif;
  margin: 0;
  padding-top: 37px; // ~ logo height in the header
}

.content {
  font-size: 9pt;
}

#header,
#footer {
  position: fixed;
  left: 0;
  right: 0;
  color: #aaa;
  font-size: 0.9em;
}

#header {
  top: 0;
  border-bottom: 0.1pt solid #aaa;

  .title {
    text-align: left;
  }

  img {
    margin-bottom: 3px;
  }
}

#footer {
  bottom: 0;
  padding-top: 2em;
}

.content table {
  margin-bottom: 3em;
}

#header table,
#footer table {
  width: 100%;
  border-collapse: collapse;
  border: none;
}

#header td,
#header th,
#footer td,
#footer th {
  padding: 0;
  width: 50%;
}

.page-number {
  padding-top: 0.5em;
  border-top: 0.1pt solid #aaa;
  text-align: center;
}

.page-number:before {
  content: "Page " counter(page);
}

hr {
  page-break-after: always;
  border: 0;
}

// General style
.state-icons,
.overview-performance-data,
.controls,
.dontprint, // Compat only, use dont-print instead
.dont-print {
  display: none !important;
}
