<?php

use Icinga\Web\Navigation\ConfigMenu;
use Icinga\Web\Widget\SearchDashboard;

$searchDashboard = new SearchDashboard();
$searchDashboard->setUser($this->Auth()->getUser());

if ($searchDashboard->search('dummy')->getPane('search')->hasDashlets()): ?>
    <form action="<?= $this->href('search') ?>" method="get" role="search" class="search-control">
        <input type="text" name="q" id="search" class="search search-input" required
               placeholder="<?= $this->translate('Search') ?> &hellip;"
               autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false">
        <button class="search-reset icon-cancel" type="reset"></button>
    </form>
<?php endif; ?>
<?= $menuRenderer->setCssClass('primary-nav')->setElementTag('nav')->setHeading(t('Navigation')); ?>
<nav class="config-menu">
    <?= new ConfigMenu() ?>
</nav>
