/*! Icinga Web 2 | (c) 2015 Icinga Development Team | GPLv2+ */

#about {
  &.content {
    display: flex;
    flex-direction: column;
    align-items: center;
  }

  section {
    width: auto;

    > * {
      margin-bottom: 2em;
    }
  }

  h2 {
    margin: 0;
  }

  .name-value-table {
    th {
      width: 100%;
    }

    th,
    td {
      white-space: nowrap;

      a {
        color: @icinga-blue
      }
    }
  }

  section:not(:last-child),
  .icinga-logo {
    margin-bottom: 2em;
  }

  .external-links {
    .rounded-corners();
    border: 1px solid @gray-light;
    display: flex;
    padding: .5em 0;
    overflow: hidden;

    .col {
      flex: 1 1 auto;
      text-align: center;
      font-size: 12/14em;
    }

    .col:not(:last-child) {
      border-right: 1px solid @gray-light;
    }

    a {
      display: block;
      padding: .75em 1em;
      margin: -7/12em 0;
    }

    a:hover {
      text-decoration: none;
      background: @gray-light;
    }

    i {
      font-size: 2*14/12em;
      opacity: .8;
      margin-bottom: .25em;
      display: block;

      &:before {
        margin-right: 0;
      }
    }
  }

  footer {
    margin-top: auto;
    align-self: stretch;
    display: flex;
    justify-content: space-between;
    align-items: baseline;

    a {
      i {
        font-size: 2em;
      }

      &:hover {
        opacity: .6;
      }
    }
  }
}
