<div class="controls">
    <?= $tabs ?>
</div>
<div class="content">
  <?php if ($this->auth()->hasPermission('config/access-control/users')): ?>
    <h1><?= $this->translate('User Backends') ?></h1>
    <?= $this->qlink(
        $this->translate('Create a New User Backend') ,
        'config/createuserbackend',
        null,
        array(
            'class'             => 'button-link',
            'data-base-target'  => '_next',
            'icon'              => 'plus',
            'title'             => $this->translate('Create a new user backend')
        )
    ) ?>
    <?= $form ?>
  <?php endif ?>

  <?php if ($this->auth()->hasPermission('config/access-control/groups')): ?>
    <h1><?= $this->translate('User Group Backends') ?></h1>
    <?= $this->qlink(
        $this->translate('Create a New User Group Backend') ,
        'usergroupbackend/create',
        null,
        array(
            'class'             => 'button-link',
            'data-base-target'  => '_next',
            'icon'              => 'plus',
            'title'             => $this->translate('Create a new user group backend')
        )
    ) ?>
<?php if (! count($backendNames)) { return; } ?>
    <table class="table-row-selectable common-table" data-base-target="_next">
    <thead>
    <tr>
        <th><?= $this->translate('Backend') ?></th>
        <th></th>
    </tr>
    </thead>
    <tbody>
<?php foreach ($backendNames as $backendName => $config):
    $type = $config->get('backend');
?>
    <tr>
        <td>
            <?= $this->qlink(
                $backendName,
                'usergroupbackend/edit',
                array('backend' => $backendName),
                array(
                    'icon'  => $type === 'external' ? 'magic' : ($type === 'ldap' || $type === 'msldap' ? 'sitemap' : 'database'),
                    'title' => sprintf($this->translate('Edit user group backend %s'), $backendName)
                )
            ); ?>
        </td>
        <td class="icon-col text-right">
            <?= $this->qlink(
                null,
                'usergroupbackend/remove',
                array('backend' => $backendName),
                array(
                    'class' => 'action-link',
                    'icon'  => 'cancel',
                    'title' => sprintf($this->translate('Remove user group backend %s'), $backendName)
                )
            ) ?>
        </td>
    </tr>
    <?php endforeach ?>
    </tbody>
    </table>
  <?php endif ?>
</div>
