<table>
    <tbody>
    <?php
    foreach ($comments as $i => $comment):
        if ($i === 5) {
            break;
        }
    ?>
    <tr>
        <td class="icon-col">
            <?= $this->partial('partials/comment/comment-description.phtml', array('comment' => $comment)) ?>
        </td>
        <td>
            <?= $this->partial('partials/comment/comment-detail.phtml', array('comment' => $comment)) ?>
        </td>
    </tr>
    <?php endforeach ?>
    </tbody>
</table>
<?php if ($comments->count() > 5): ?>
<p>
    <?= $this->qlink(
        sprintf($this->translate('List all %d comments'), $comments->count()),
        $listAllLink,
        null,
        array(
            'data-base-target'  => '_next',
            'icon'              => 'down-open'
        )
    ) ?>
</p>
<?php endif ?>
