<table>
  <tr>
    <td class="icon-col">
        <?= $this->render('partials/comment/comment-description.phtml') ?>
    </td>
    <td>
        <?= $this->render('partials/comment/comment-detail.phtml') ?>
    </td>
  </tr>
</table>
